@@ -1,6 +1,6 @@
 require_relative "utils"
 
-if defined?(OpenSSL)
+if defined?(OpenSSL::TestUtils)
 
 class OpenSSL::TestSSL < OpenSSL::SSLTestCase
 
@@ -19,6 +19,13 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
                  ctx.options & OpenSSL::SSL::OP_NO_COMPRESSION)
   end if defined?(OpenSSL::SSL::OP_NO_COMPRESSION)
 
+  def test_ctx_setup_with_extra_chain_cert
+    ctx = OpenSSL::SSL::SSLContext.new
+    ctx.extra_chain_cert = [@ca_cert, @cli_cert]
+    assert_equal(ctx.setup, true)
+    assert_equal(ctx.setup, nil)
+  end
+
   def test_not_started_session
     skip "non socket argument of SSLSocket.new is not supported on this platform" if /mswin|mingw/ =~ RUBY_PLATFORM
     open(__FILE__) do |f|
@@ -27,7 +34,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
   end
 
   def test_ssl_gets
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true) { |server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true) { |server, port|
       server_connect(port) { |ssl|
         ssl.write "abc\n"
         IO.select [ssl]
@@ -41,7 +48,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
   end
 
   def test_ssl_read_nonblock
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true) { |server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true) { |server, port|
       server_connect(port) { |ssl|
         assert_raise(IO::WaitReadable) { ssl.read_nonblock(100) }
         ssl.write("abc\n")
@@ -54,7 +61,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
   end
 
   def test_connect_and_close
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ssl = OpenSSL::SSL::SSLSocket.new(sock)
       assert(ssl.connect)
@@ -72,19 +79,23 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
   end
 
   def test_read_and_write
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       server_connect(port) { |ssl|
         # syswrite and sysread
         ITERATIONS.times{|i|
           str = "x" * 100 + "\n"
           ssl.syswrite(str)
-          assert_equal(str, ssl.sysread(str.size))
+          newstr = ''
+          newstr << ssl.sysread(str.size - newstr.size) until newstr.size == str.size
+          assert_equal(str, newstr)
 
           str = "x" * i * 100 + "\n"
           buf = ""
           ssl.syswrite(str)
           assert_equal(buf.object_id, ssl.sysread(str.size, buf).object_id)
-          assert_equal(str, buf)
+          newstr = buf
+          newstr << ssl.sysread(str.size - newstr.size) until newstr.size == str.size
+          assert_equal(str, newstr)
         }
 
         # puts and gets
@@ -115,15 +126,25 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     }
   end
 
-  def test_client_auth
+  def test_client_auth_failure
     vflag = OpenSSL::SSL::VERIFY_PEER|OpenSSL::SSL::VERIFY_FAIL_IF_NO_PEER_CERT
-    start_server(PORT, vflag, true){|server, port|
+    start_server(vflag, true, :ignore_listener_error => true){|server, port|
       assert_raise(OpenSSL::SSL::SSLError, Errno::ECONNRESET){
         sock = TCPSocket.new("127.0.0.1", port)
         ssl = OpenSSL::SSL::SSLSocket.new(sock)
-        ssl.connect
+        ssl.sync_close = true
+        begin
+          ssl.connect
+        ensure
+          ssl.close
+        end
       }
+    }
+  end
 
+  def test_client_auth_success
+    vflag = OpenSSL::SSL::VERIFY_PEER|OpenSSL::SSL::VERIFY_FAIL_IF_NO_PEER_CERT
+    start_server(vflag, true){|server, port|
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.key = @cli_key
       ctx.cert = @cli_cert
@@ -154,7 +175,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     end
 
     vflag = OpenSSL::SSL::VERIFY_PEER|OpenSSL::SSL::VERIFY_FAIL_IF_NO_PEER_CERT
-    start_server(PORT, vflag, true, :ctx_proc => ctx_proc){|server, port|
+    start_server(vflag, true, :ctx_proc => ctx_proc){|server, port|
       ctx = OpenSSL::SSL::SSLContext.new
       client_ca_from_server = nil
       ctx.client_cert_cb = Proc.new do |sslconn|
@@ -165,33 +186,51 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     }
   end
 
+  def test_read_nonblock_without_session
+    OpenSSL::TestUtils.silent do
+      start_server(OpenSSL::SSL::VERIFY_NONE, false){|server, port|
+        sock = TCPSocket.new("127.0.0.1", port)
+        ssl = OpenSSL::SSL::SSLSocket.new(sock)
+        ssl.sync_close = true
+
+        assert_equal :wait_readable, ssl.read_nonblock(100, exception: false)
+        ssl.write("abc\n")
+        IO.select [ssl]
+        assert_equal('a', ssl.read_nonblock(1))
+        assert_equal("bc\n", ssl.read_nonblock(100))
+        assert_equal :wait_readable, ssl.read_nonblock(100, exception: false)
+        ssl.close
+      }
+    end
+  end
+
   def test_starttls
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, false){|server, port|
-      sock = TCPSocket.new("127.0.0.1", port)
-      ssl = OpenSSL::SSL::SSLSocket.new(sock)
-      ssl.sync_close = true
-      str = "x" * 1000 + "\n"
+    OpenSSL::TestUtils.silent do
+      start_server(OpenSSL::SSL::VERIFY_NONE, false){|server, port|
+        sock = TCPSocket.new("127.0.0.1", port)
+        ssl = OpenSSL::SSL::SSLSocket.new(sock)
+        ssl.sync_close = true
+        str = "x" * 1000 + "\n"
 
-      OpenSSL::TestUtils.silent do
         ITERATIONS.times{
           ssl.puts(str)
           assert_equal(str, ssl.gets)
         }
         starttls(ssl)
-      end
 
-      ITERATIONS.times{
-        ssl.puts(str)
-        assert_equal(str, ssl.gets)
+        ITERATIONS.times{
+          ssl.puts(str)
+          assert_equal(str, ssl.gets)
+        }
+
+        ssl.close
       }
-
-      ssl.close
-    }
+    end
   end
 
   def test_parallel
     GC.start
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       ssls = []
       10.times{
         sock = TCPSocket.new("127.0.0.1", port)
@@ -212,14 +251,21 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
   end
 
   def test_verify_result
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ignore_listener_error => true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.set_params
       ssl = OpenSSL::SSL::SSLSocket.new(sock, ctx)
-      assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
-      assert_equal(OpenSSL::X509::V_ERR_SELF_SIGNED_CERT_IN_CHAIN, ssl.verify_result)
+      ssl.sync_close = true
+      begin
+        assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
+        assert_equal(OpenSSL::X509::V_ERR_SELF_SIGNED_CERT_IN_CHAIN, ssl.verify_result)
+      ensure
+        ssl.close
+      end
+    }
 
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.set_params(
@@ -229,9 +275,16 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
         end
       )
       ssl = OpenSSL::SSL::SSLSocket.new(sock, ctx)
-      ssl.connect
-      assert_equal(OpenSSL::X509::V_OK, ssl.verify_result)
+      ssl.sync_close = true
+      begin
+        ssl.connect
+        assert_equal(OpenSSL::X509::V_OK, ssl.verify_result)
+      ensure
+        ssl.close
+      end
+    }
 
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ignore_listener_error => true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.set_params(
@@ -241,13 +294,18 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
         end
       )
       ssl = OpenSSL::SSL::SSLSocket.new(sock, ctx)
-      assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
-      assert_equal(OpenSSL::X509::V_ERR_APPLICATION_VERIFICATION, ssl.verify_result)
+      ssl.sync_close = true
+      begin
+        assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
+        assert_equal(OpenSSL::X509::V_ERR_APPLICATION_VERIFICATION, ssl.verify_result)
+      ensure
+        ssl.close
+      end
     }
   end
 
   def test_exception_in_verify_callback_is_ignored
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ignore_listener_error => true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.set_params(
@@ -257,17 +315,21 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
         end
       )
       ssl = OpenSSL::SSL::SSLSocket.new(sock, ctx)
-      OpenSSL::TestUtils.silent do
-        # SSLError, not RuntimeError
-        assert_raise(OpenSSL::SSL::SSLError) { ssl.connect }
+      ssl.sync_close = true
+      begin
+        OpenSSL::TestUtils.silent do
+          # SSLError, not RuntimeError
+          assert_raise(OpenSSL::SSL::SSLError) { ssl.connect }
+        end
+        assert_equal(OpenSSL::X509::V_ERR_CERT_REJECTED, ssl.verify_result)
+      ensure
+        ssl.close
       end
-      assert_equal(OpenSSL::X509::V_ERR_CERT_REJECTED, ssl.verify_result)
-      ssl.close
     }
   end
 
   def test_sslctx_set_params
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ignore_listener_error => true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ctx = OpenSSL::SSL::SSLContext.new
       ctx.set_params
@@ -279,29 +341,20 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
       assert(ciphers_names.all?{|v| /ADH/ !~ v })
       assert(ciphers_versions.all?{|v| /SSLv2/ !~ v })
       ssl = OpenSSL::SSL::SSLSocket.new(sock, ctx)
-      assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
-      assert_equal(OpenSSL::X509::V_ERR_SELF_SIGNED_CERT_IN_CHAIN, ssl.verify_result)
-    }
-  end
-
-  def test_post_connect_check_with_anon_ciphers
-    sslerr = OpenSSL::SSL::SSLError
-
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true, {use_anon_cipher: true}){|server, port|
-      ctx = OpenSSL::SSL::SSLContext.new
-      ctx.ciphers = "aNULL"
-      server_connect(port, ctx) { |ssl|
-        msg = "Peer verification enabled, but no certificate received. Anonymous cipher suite " \
-          "ADH-AES256-GCM-SHA384 was negotiated. Anonymous suites must be disabled to use peer verification."
-        assert_raise_with_message(sslerr,msg){ssl.post_connection_check("localhost.localdomain")}
-      }
+      ssl.sync_close = true
+      begin
+        assert_raise(OpenSSL::SSL::SSLError){ ssl.connect }
+        assert_equal(OpenSSL::X509::V_ERR_SELF_SIGNED_CERT_IN_CHAIN, ssl.verify_result)
+      ensure
+        ssl.close
+      end
     }
   end
 
   def test_post_connection_check
     sslerr = OpenSSL::SSL::SSLError
 
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       server_connect(port) { |ssl|
         assert_raise(sslerr){ssl.post_connection_check("localhost.localdomain")}
         assert_raise(sslerr){ssl.post_connection_check("127.0.0.1")}
@@ -324,7 +377,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     ]
     @svr_cert = issue_cert(@svr, @svr_key, 4, now, now+1800, exts,
                            @ca_cert, @ca_key, OpenSSL::Digest::SHA1.new)
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       server_connect(port) { |ssl|
         assert(ssl.post_connection_check("localhost.localdomain"))
         assert(ssl.post_connection_check("127.0.0.1"))
@@ -346,7 +399,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     ]
     @svr_cert = issue_cert(@svr, @svr_key, 5, now, now+1800, exts,
                            @ca_cert, @ca_key, OpenSSL::Digest::SHA1.new)
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       server_connect(port) { |ssl|
         assert(ssl.post_connection_check("localhost.localdomain"))
         assert_raise(sslerr){ssl.post_connection_check("127.0.0.1")}
@@ -373,156 +426,6 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
     end
   end
 
-  def test_verify_hostname
-    assert_equal(true,  OpenSSL::SSL.verify_hostname("www.example.com", "*.example.com"))
-    assert_equal(false, OpenSSL::SSL.verify_hostname("www.subdomain.example.com", "*.example.com"))
-  end
-
-  def test_verify_wildcard
-    assert_equal(false, OpenSSL::SSL.verify_wildcard("foo", "x*"))
-    assert_equal(true,  OpenSSL::SSL.verify_wildcard("foo", "foo"))
-    assert_equal(true,  OpenSSL::SSL.verify_wildcard("foo", "f*"))
-    assert_equal(true,  OpenSSL::SSL.verify_wildcard("foo", "*"))
-    assert_equal(false, OpenSSL::SSL.verify_wildcard("abc*bcd", "abcd"))
-    assert_equal(false, OpenSSL::SSL.verify_wildcard("xn--qdk4b9b", "x*"))
-    assert_equal(false, OpenSSL::SSL.verify_wildcard("xn--qdk4b9b", "*--qdk4b9b"))
-    assert_equal(true,  OpenSSL::SSL.verify_wildcard("xn--qdk4b9b", "xn--qdk4b9b"))
-  end
-
-  # Comments in this test is excerpted from http://tools.ietf.org/html/rfc6125#page-27
-  def test_post_connection_check_wildcard_san
-    # case-insensitive ASCII comparison
-    # RFC 6125, section 6.4.1
-    #
-    # "..matching of the reference identifier against the presented identifier
-    # is performed by comparing the set of domain name labels using a
-    # case-insensitive ASCII comparison, as clarified by [DNS-CASE] (e.g.,
-    # "WWW.Example.Com" would be lower-cased to "www.example.com" for
-    # comparison purposes)
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*.example.com'), 'www.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*.Example.COM'), 'www.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*.example.com'), 'WWW.Example.COM'))
-    # 1.  The client SHOULD NOT attempt to match a presented identifier in
-    #     which the wildcard character comprises a label other than the
-    #     left-most label (e.g., do not match bar.*.example.net).
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:www.*.com'), 'www.example.com'))
-    # 2.  If the wildcard character is the only character of the left-most
-    #     label in the presented identifier, the client SHOULD NOT compare
-    #     against anything but the left-most label of the reference
-    #     identifier (e.g., *.example.com would match foo.example.com but
-    #     not bar.foo.example.com or example.com).
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*.example.com'), 'foo.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*.example.com'), 'bar.foo.example.com'))
-    # 3.  The client MAY match a presented identifier in which the wildcard
-    #     character is not the only character of the label (e.g.,
-    #     baz*.example.net and *baz.example.net and b*z.example.net would
-    #     be taken to match baz1.example.net and foobaz.example.net and
-    #     buzz.example.net, respectively).  ...
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:baz*.example.com'), 'baz1.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*baz.example.com'), 'foobaz.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:b*z.example.com'), 'buzz.example.com'))
-    # Section 6.4.3 of RFC6125 states that client should NOT match identifier
-    # where wildcard is other than left-most label.
-    #
-    # Also implicitly mentions the wildcard character only in singular form,
-    # and discourages matching against more than one wildcard.
-    #
-    # See RFC 6125, section 7.2, subitem 2.
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*b*.example.com'), 'abc.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*b*.example.com'), 'ab.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:*b*.example.com'), 'bc.example.com'))
-    #                                ...  However, the client SHOULD NOT
-    #   attempt to match a presented identifier where the wildcard
-    #   character is embedded within an A-label or U-label [IDNA-DEFS] of
-    #   an internationalized domain name [IDNA-PROTO].
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:xn*.example.com'), 'xn1ca.example.com'))
-    # part of A-label
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:xn--*.example.com'), 'xn--1ca.example.com'))
-    # part of U-label
-    # dNSName in RFC5280 is an IA5String so U-label should NOT be allowed
-    # regardless of wildcard.
-    #
-    # See Section 7.2 of RFC 5280:
-    #   IA5String is limited to the set of ASCII characters.
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_san('DNS:á*.example.com'), 'á1.example.com'))
-  end
-
-  def test_post_connection_check_wildcard_cn
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*.example.com'), 'www.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*.Example.COM'), 'www.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*.example.com'), 'WWW.Example.COM'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('www.*.com'), 'www.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*.example.com'), 'foo.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*.example.com'), 'bar.foo.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('baz*.example.com'), 'baz1.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*baz.example.com'), 'foobaz.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('b*z.example.com'), 'buzz.example.com'))
-    # Section 6.4.3 of RFC6125 states that client should NOT match identifier
-    # where wildcard is other than left-most label.
-    #
-    # Also implicitly mentions the wildcard character only in singular form,
-    # and discourages matching against more than one wildcard.
-    #
-    # See RFC 6125, section 7.2, subitem 2.
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*b*.example.com'), 'abc.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*b*.example.com'), 'ab.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('*b*.example.com'), 'bc.example.com'))
-    assert_equal(true, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('xn*.example.com'), 'xn1ca.example.com'))
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('xn--*.example.com'), 'xn--1ca.example.com'))
-    # part of U-label
-    # Subject in RFC5280 states case-insensitive ASCII comparison.
-    #
-    # See Section 7.2 of RFC 5280:
-    #   IA5String is limited to the set of ASCII characters.
-    assert_equal(false, OpenSSL::SSL.verify_certificate_identity(
-      create_cert_with_name('á*.example.com'), 'á1.example.com'))
-  end
-
-  def create_cert_with_san(san)
-    ef = OpenSSL::X509::ExtensionFactory.new
-    cert = OpenSSL::X509::Certificate.new
-    cert.subject = OpenSSL::X509::Name.parse("/DC=some/DC=site/CN=Some Site")
-    ext = ef.create_ext('subjectAltName', san)
-    cert.add_extension(ext)
-    cert
-  end
-
-  def create_cert_with_name(name)
-    cert = OpenSSL::X509::Certificate.new
-    cert.subject = OpenSSL::X509::Name.new([['DC', 'some'], ['DC', 'site'], ['CN', name]])
-    cert
-  end
-
-
   # Create NULL byte SAN certificate
   def create_null_byte_SAN_certificate(critical = false)
     ef = OpenSSL::X509::ExtensionFactory.new
@@ -562,7 +465,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
       readwrite_loop(ctx, ssl)
     end
 
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true, :ctx_proc => ctx_proc, :server_proc => server_proc) do |server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ctx_proc => ctx_proc, :server_proc => server_proc) do |server, port|
       2.times do |i|
         ctx = OpenSSL::SSL::SSLContext.new
         if defined?(OpenSSL::SSL::OP_NO_TICKET)
@@ -595,7 +498,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
         assert_equal(num_written, raw_size)
         ssl.close
       }
-      start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true, :server_proc => server_proc){|server, port|
+      start_server(OpenSSL::SSL::VERIFY_NONE, true, :server_proc => server_proc){|server, port|
         server_connect(port) { |ssl|
           str = auml * i
           num_written = ssl.write(str)
@@ -611,7 +514,7 @@ class OpenSSL::TestSSL < OpenSSL::SSLTestCase
       # But it also degrades gracefully, so keep it
       ctx.options = OpenSSL::SSL::OP_ALL
     }
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true, :ctx_proc => ctx_proc){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true, :ctx_proc => ctx_proc){|server, port|
       server_connect(port) { |ssl|
         ssl.puts('hello')
         assert_equal("hello\n", ssl.gets)
@@ -797,7 +700,7 @@ end
 
   def test_invalid_shutdown_by_gc
     assert_nothing_raised {
-      start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+      start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
         10.times {
           sock = TCPSocket.new("127.0.0.1", port)
           ssl = OpenSSL::SSL::SSLSocket.new(sock)
@@ -810,7 +713,7 @@ end
   end
 
   def test_close_after_socket_close
-    start_server(PORT, OpenSSL::SSL::VERIFY_NONE, true){|server, port|
+    start_server(OpenSSL::SSL::VERIFY_NONE, true){|server, port|
       sock = TCPSocket.new("127.0.0.1", port)
       ssl = OpenSSL::SSL::SSLSocket.new(sock)
       ssl.sync_close = true
@@ -822,6 +725,15 @@ end
     }
   end
 
+  def test_sync_close_without_connect
+    Socket.open(:INET, :STREAM) {|s|
+      ssl = OpenSSL::SSL::SSLSocket.new(s)
+      ssl.sync_close = true
+      ssl.close
+      assert(s.closed?)
+    }
+  end
+
   private
 
   def start_server_version(version, ctx_proc=nil, server_proc=nil, &blk)
@@ -830,11 +742,11 @@ end
       ctx_proc.call(ctx) if ctx_proc
     }
     start_server(
-      PORT,
       OpenSSL::SSL::VERIFY_NONE,
       true,
       :ctx_proc => ctx_wrap,
       :server_proc => server_proc,
+      :ignore_listener_error => true,
       &blk
     )
   end
@@ -844,9 +756,13 @@ end
     ssl = ctx ? OpenSSL::SSL::SSLSocket.new(sock, ctx) : OpenSSL::SSL::SSLSocket.new(sock)
     ssl.sync_close = true
     ssl.connect
-    yield ssl if block_given?
+    yield ssl
   ensure
-    ssl.close
+    if ssl
+      ssl.close
+    elsif sock
+      sock.close
+    end
   end
 end
 
